/*
    Synth Lua - Formant speech synthesizer
    Copyright (C) 2013 Belousov Oleg <belousov.oleg@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Cordic in 32 bit signed fixed point math
// Function is valid for arguments in range -pi/2 -- pi/2
// for values pi/2--pi: value = half_pi-(theta-half_pi) and similarly for values -pi---pi/2

#include "cordic.h"

#define CORDIC_NTAB 32

static int cordic_ctab [] = {
    0x3243F6A8, 0x1DAC6705, 0x0FADBAFC, 0x07F56EA6, 0x03FEAB76, 0x01FFD55B,
    0x00FFFAAA, 0x007FFF55, 0x003FFFEA, 0x001FFFFD, 0x000FFFFF, 0x0007FFFF, 0x0003FFFF,
    0x0001FFFF, 0x0000FFFF, 0x00007FFF, 0x00003FFF, 0x00001FFF, 0x00000FFF, 0x000007FF,
    0x000003FF, 0x000001FF, 0x000000FF, 0x0000007F, 0x0000003F, 0x0000001F, 0x0000000F,
    0x00000008, 0x00000004, 0x00000002, 0x00000001, 0x00000000
};

static int scale(int x) {
    //                       12345678901234567
    // (0.60725 * 0.984) = 0.10011000111101111

    return
	(x >> 1) +
	(x >> 4) +
	(x >> 5) +
	(x >> 9) +
	(x >> 10) +
	(x >> 11) +
	(x >> 12) +
	(x >> 14) +
	(x >> 17);
}

void cordic(int theta, int *s, int *c, int n) {
    int	k, d, tx, ty, tz;
    int	z = theta;
    int x, y;

    x = scale(*s);
    y = scale(*c);

    n = (n > CORDIC_NTAB) ? CORDIC_NTAB : n;

    for (k = 0; k < n; ++k) {
	d = z>=0 ? 0 : -1;

	tx = x - (((y >> k) ^ d) - d);
	ty = y + (((x >> k) ^ d) - d);
	tz = z - ((cordic_ctab[k] ^ d) - d);

	x = tx;
	y = ty;
	z = tz;
    }

    *s = x;
    *c = y;
}
