/*
    Synth Lua - Formant speech synthesizer
    Copyright (C) 2013 Belousov Oleg <belousov.oleg@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "tract.h"
#include "formant.h"
#include "cordic.h"

typedef struct {
    int theta;
    int fx;
    int fy;
} formant_t;

static formant_t f1 = { 0, 0, 0 };
static formant_t f2 = { 0, 0, 0 };
static formant_t f3 = { 0, 0, 0 };
static formant_t f4 = { 0, 0, 0 };

void formant(formant_t *f, int *x, int *y) {
    *x += f->fx;
    *y += f->fy;

    cordic(f->theta, y, x, 32);

    f->fx = *x;
    f->fy = *y;
}

void formant_set(int ff1, int ff2, int ff3, int ff4) {
    f1.theta = CORDIC_HALF_PI / RATE * 4 * ff1;
    f2.theta = CORDIC_HALF_PI / RATE * 4 * ff2;
    f3.theta = CORDIC_HALF_PI / RATE * 4 * ff3;
    f4.theta = CORDIC_HALF_PI / RATE * 4 * ff4;
}

void formant_render(int *x, int *y) {
    formant(&f1, x, y);
    formant(&f2, x, y);
    formant(&f3, x, y);
    formant(&f4, x, y);
}
