/*
    Synth Lua - Formant speech synthesizer
    Copyright (C) 2013 Belousov Oleg <belousov.oleg@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "lua.h"
#include "lauxlib.h"

#include "tract.h"
#include "noise.h"
#include "cordic.h"
#include "formant.h"
#include "glotal.h"
#include "nasal.h"
#include "frication.h"

static int Lrate(lua_State *L) {
    tract_rate = lua_tonumber(L, 1);
    return 0;
}

static int Lvp(lua_State *L) {
    tract_vp = lua_tonumber(L, 1);
    return 0;
}

static int Lvr(lua_State *L) {
    tract_vr = lua_tonumber(L, 1);
    return 0;
}

static int Lgf(lua_State *L) {
    tract_gf = lua_tonumber(L, 1);
    return 0;
}

static int Lga(lua_State *L) {
    tract_ga = lua_tonumber(L, 1);
    return 0;
}

static int Laa(lua_State *L) {
    tract_aa = lua_tonumber(L, 1);
    return 0;
}

static int Lf(lua_State *L) {
    int f1  = lua_tonumber(L, 1);
    int f2  = lua_tonumber(L, 2);
    int f3  = lua_tonumber(L, 3);
    int f4  = lua_tonumber(L, 4);

    formant_set(f1, f2, f3, f4);

    return 0;
}

static int Lnf(lua_State *L) {
    tract_nf = lua_tonumber(L, 1);
    return 0;
}

static int Lna(lua_State *L) {
    tract_na = lua_tonumber(L, 1);
    return 0;
}

static int Lff(lua_State *L) {
    tract_ff = lua_tonumber(L, 1);
    return 0;
}

static int Lfa(lua_State *L) {
    tract_fa = lua_tonumber(L, 1);
    return 0;
}

static int Lrender(lua_State *L) {
    int scale = lua_tonumber(L, 1);
    int x = tract_render() / scale;

    lua_pushnumber(L, x);
    return 1;
}

static const luaL_Reg tab_funcs[] = {
    { "rate",	Lrate },
    { "vp",	Lvp },
    { "vr",	Lvr },
    { "gf",	Lgf },
    { "ga",	Lga },
    { "aa",	Laa },
    { "f",	Lf },
    { "nf",	Lnf },
    { "na",	Lna },
    { "ff",	Lff },
    { "fa",	Lfa },
    { "render",	Lrender },
    { NULL,	NULL }
};

LUALIB_API int luaopen_tract(lua_State *L) {
    luaL_newlib(L, tab_funcs);
    lua_setglobal(L, "tract");

    return 0;
}
