#!/usr/local/bin/lua

require "tract"

local f = io.open("test.raw", "wb")
local rate = 44100

tract.rate(rate)
tract.vp(1)	-- Set vibrato pinch
tract.vr(5)	-- Set vibrato rate
tract.gf(90)	-- Set glotal freq

local t_m = {
    ga = 20,
    aa = 250,
    f1 = 170, f2 = 1100, f3 = 2600, f4 = 3200,
    nf = 2900, na = 200
}

local t_o = {
    ga = 8,
    aa = 250,
    f1 = 750, f2 = 1050, f3 = 2400, f4 = 3200
}

local t_n = {
    ga = 40,
    aa = 250,
    f1 = 200, f2 = 1550, f3 = 2500, f4 = 4500,
    nf = 1550, na = 200
}

local t_s = {
    aa = 5,
    f1 = 2057, f2 = 3170, f3 = 4079, f4 = 4000,
    ff = 3170, fa = 30
}

local t_t = {
    aa = 5,
    f1 = 2057, f2 = 3170, f3 = 4079, f4 = 4000,
    ff = 3125, fa = 90
}

local t_r = {
    ga = 5,
    aa = 100,
    f1 = 580, f2 = 1200, f3 = 1500, f4 = 4700
}

---

local cur
local speed = 1

local function render()
    local out = tract.render(1024*1024*8)

    f:write(string.format("%c", out))
end

local function fade_in(item)
    local len = item[1] / 1000 * rate * speed
    local phone = item[3]

    cur = phone

    if phone.f1 then
	tract.f(phone.f1, phone.f2, phone.f3, phone.f4)
    end

    tract.na(phone.na or 0)
    tract.nf(phone.nf or 0)
    tract.ff(phone.ff or 0)

    for i = 0,len do
	local scale = i/len

	if phone.aa then
	    tract.aa(phone.aa * scale)
	end

	if phone.ga then
	    tract.ga(phone.ga * scale)
	end

	if phone.fa then
	    tract.fa(phone.fa * scale)
	end

	render()
    end
end

local function transition(item)
    local len = item[1] / 1000 * rate * speed
    local from = cur
    local to = item[3]

    for i = 0,len do
	local scale = i/len

	local function inter(x, y)
	    x = x or 0
	    y = y or 0

	    return x + (y-x) * scale
	end

	tract.aa(inter(from.aa, to.aa))
	tract.ga(inter(from.ga, to.ga))
	tract.na(inter(from.na, to.na))
	tract.nf(inter(from.nf, to.nf))
	tract.fa(inter(from.fa, to.fa))
	tract.ff(inter(from.ff, to.ff))

	if from.f1 then
	    tract.f(
		from.f1 + (to.f1 - from.f1) * scale,
		from.f2 + (to.f2 - from.f2) * scale,
		from.f3 + (to.f3 - from.f3) * scale,
		from.f4 + (to.f4 - from.f4) * scale
	    )
	end

	render()
    end

    cur = to
end

local function sustain(item)
    local len = item[1] / 1000 * rate * speed

    for i = 0,len do
	render()
    end
end

local function pause(item)
    local len = item[1] / 1000 * rate

    tract.aa(0)
    tract.ga(0)
    tract.fa(0)

    for i = 0,len do
	render()
    end
end

local function fade_out(item)
    local len = item[1] / 1000 * rate * speed
    local phone = cur

    if phone.f1 then
	tract.f(phone.f1, phone.f2, phone.f3, phone.f4)
    end

    tract.na(phone.na or 0)
    tract.nf(phone.nf or 0)
    tract.ff(phone.ff or 0)

    for i = 0,len do
	local scale = 1 - i/len

	if phone.aa then
	    tract.aa(phone.aa * scale)
	end

	if phone.ga then
	    tract.ga(phone.ga * scale)
	end

	if phone.fa then
	    tract.fa(phone.fa * scale)
	end

	render()
    end
end

local function say(s)
    for n,item in ipairs(s) do
	local func = item[2]

	if func then
	    func(item)
	end
    end
end

---

local monster = {
    { 75, fade_in, 	t_m },
    { 25, transition,	t_o },
    { 75, sustain },

    { 75, transition,	t_n },
    { 75, sustain },
    { 50, fade_out },

    { 75, fade_in,	t_s },
    { 35, fade_out },

    { 25, fade_in,	t_t },
    { 10, fade_out },

    { 15, fade_in,	t_r },
    { 15, fade_out },
    { 15, fade_in,	t_r },
    { 15, fade_out },
    { 15, fade_in,	t_r },
    { 15, fade_out },
    { 15, fade_in,	t_r },
    { 15, fade_out },

    { 250, pause }
}

for i = 1,15 do
    tract.gf(40 + math.random(0, 20) * 10)
    speed = math.random(0.5, 2)

    say(monster)
end
